@@ -12,9 +12,10 @@ of the HostsEntry class.
 """
 
 import sys
+
 try:
     from urllib.request import urlopen
-except ImportError:
+except ImportError:  # pragma: no cover
     from urllib2 import urlopen
 from python_hosts.utils import (is_ipv4, is_ipv6, is_readable, valid_hostnames,
                                 dedupe_list)
@@ -66,10 +67,11 @@ class HostsEntry(object):
         self.names = names
 
     def __repr__(self):
-        return "HostsEntry(entry_type=%r, address=%r, comment=%r, names=%r)" % (self.entry_type,
-                                                                                self.address,
-                                                                                self.comment,
-                                                                                self.names)
+        return "HostsEntry(entry_type=\'{0}\', address=\'{1}\', " \
+               "comment={2}, names={3})".format(self.entry_type,
+                                                self.address,
+                                                self.comment,
+                                                self.names)
 
     def __str__(self):
         if self.entry_type in ('ipv4', 'ipv6'):
@@ -108,16 +110,14 @@ class HostsEntry(object):
         :return: An instance of HostsEntry
         """
         line_parts = entry.strip().split()
-        if is_ipv4(line_parts[0]):
-            if valid_hostnames(line_parts[1:]):
-                return HostsEntry(entry_type='ipv4',
-                                  address=line_parts[0],
-                                  names=line_parts[1:])
-        elif is_ipv6(line_parts[0]):
-            if valid_hostnames(line_parts[1:]):
-                return HostsEntry(entry_type='ipv6',
-                                  address=line_parts[0],
-                                  names=line_parts[1:])
+        if is_ipv4(line_parts[0]) and valid_hostnames(line_parts[1:]):
+            return HostsEntry(entry_type='ipv4',
+                              address=line_parts[0],
+                              names=line_parts[1:])
+        elif is_ipv6(line_parts[0]) and valid_hostnames(line_parts[1:]):
+            return HostsEntry(entry_type='ipv6',
+                              address=line_parts[0],
+                              names=line_parts[1:])
         else:
             return False
 
@@ -141,7 +141,7 @@ class Hosts(object):
         self.populate_entries()
 
     def __repr__(self):
-        return 'Hosts(hosts_path=%r, entries=%r)' % (self.hosts_path, self.entries)
+        return 'Hosts(hosts_path=\'{0}\', entries={1})'.format(self.hosts_path, self.entries)
 
     def __str__(self):
         output = ('hosts_path={0}, '.format(self.hosts_path))
@@ -171,9 +171,10 @@ class Hosts(object):
         else:
             return '/etc/hosts'
 
-    def write(self):
+    def write(self, path=None):
         """
         Write all of the HostsEntry instances back to the hosts file
+        :param path: override the write path
         :return: Dictionary containing counts
         """
         written_count = 0
@@ -181,8 +182,12 @@ class Hosts(object):
         blanks_written = 0
         ipv4_entries_written = 0
         ipv6_entries_written = 0
+        if path:
+            output_file_path = path
+        else:
+            output_file_path = self.hosts_path
         try:
-            with open(self.hosts_path, 'w') as hosts_file:
+            with open(output_file_path, 'w') as hosts_file:
                 for written_count, line in enumerate(self.entries):
                     if line.entry_type == 'comment':
                         hosts_file.write(line.comment)
@@ -195,18 +200,18 @@ class Hosts(object):
                             "{0}\t{1}\n".format(
                                 line.address,
                                 ' '.join(line.names),
-                                )
                             )
+                        )
                         ipv4_entries_written += 1
                     if line.entry_type == 'ipv6':
                         hosts_file.write(
                             "{0}\t{1}\n".format(
                                 line.address,
-                                ' '.join(line.names),))
+                                ' '.join(line.names), ))
                         ipv6_entries_written += 1
         except:
-            raise UnableToWriteHosts
-        return {'total_written': written_count+1,
+            raise UnableToWriteHosts()
+        return {'total_written': written_count + 1,
                 'comments_written': comments_written,
                 'blanks_written': blanks_written,
                 'ipv4_entries_written': ipv4_entries_written,
@@ -248,14 +253,14 @@ class Hosts(object):
         """
         if self.entries:
             if address and name:
-                to_remove = lambda x: x.address == address and name in x.names
+                func = lambda entry: entry.address != address and name not in entry.names
             elif address:
-                to_remove = lambda x: x.address == address
+                func = lambda entry: entry.address != address
             elif name:
-                to_remove = lambda x: x.names and name in x.names
+                func = lambda entry: name not in entry.names
             else:
-                return
-            self.entries = [x for x in self.entries if not to_remove(x)]
+                raise ValueError('No address or name was specified for removal.')
+            self.entries = list(filter(func, self.entries))
 
     def import_url(self, url=None):
         """
@@ -326,6 +331,7 @@ class Hosts(object):
         """
         Add instances of HostsEntry to the instance of Hosts.
         :param entries: A list of instances of HostsEntry
+        :param force: Remove matching before adding
         :return: The counts of successes and failures
         """
         ipv4_count = 0
@@ -358,12 +364,12 @@ class Hosts(object):
                     replaced_count += 1
                     import_entries.append(entry)
             elif set(entry.names).intersection(existing_names):
-                    if not force:
-                        duplicate_count += 1
-                    else:
-                        self.remove_all_matching(name=entry.names)
-                        replaced_count += 1
-                        import_entries.append(entry)
+                if not force:
+                    duplicate_count += 1
+                else:
+                    self.remove_all_matching(name=entry.names)
+                    replaced_count += 1
+                    import_entries.append(entry)
             else:
                 import_entries.append(entry)
 
@@ -396,7 +402,7 @@ class Hosts(object):
                                                        comment=hosts_entry))
                     elif entry_type == "blank":
                         self.entries.append(HostsEntry(entry_type="blank"))
-                    elif entry_type == "ipv4" or entry_type == "ipv6":
+                    elif entry_type in ("ipv4", "ipv6"):
                         chunked_entry = hosts_entry.split()
                         stripped_name_list = [name.strip() for name in chunked_entry[1:]]
 
