@@ -1,19 +1,39 @@
 # -*- coding: utf-8 -*-
 import datetime
-import sys
 import os
+import pwd
+import sys
+
 import pytest
+
 sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
 from python_hosts import Hosts, HostsEntry, exception
 
 
+def get_username():
+    return pwd.getpwuid(os.getuid())[0]
+
+
 def test_hosts_str(tmpdir):
     """ Test that the str method returns an accurate, user readable output
     """
     hosts_file = tmpdir.mkdir("etc").join("hosts")
     hosts_file.write("6.6.6.6\texample.com\n")
     hosts = Hosts(path=hosts_file.strpath)
-    assert(str(hosts)) == "hosts_path={0}, TYPE=ipv4, ADDR=6.6.6.6, NAMES=example.com".format(hosts_file.strpath)
+    assert (str(hosts)) == "hosts_path={0}, TYPE=ipv4, ADDR=6.6.6.6, NAMES=example.com".format(hosts_file.strpath)
+
+
+def test_hosts_write_to_custom_path(tmpdir):
+    """ Test that the hosts file can be written to a different path to the one it was read from
+    """
+    hosts_file = tmpdir.mkdir("etc").join("hosts")
+    hosts_file.write("6.6.6.6\texample.com\n")
+    hosts = Hosts(path=hosts_file.strpath)
+    alternate_hosts_file = tmpdir.mkdir("tmp").join("hosts2")
+    hosts.write(path=alternate_hosts_file.strpath)
+    alternate_hosts = Hosts(path=alternate_hosts_file.strpath)
+    assert alternate_hosts.count() == 1
+    assert hosts.exists(address='6.6.6.6', names=['example.com'])
 
 
 def test_hosts_repr(tmpdir):
@@ -22,11 +42,11 @@ def test_hosts_repr(tmpdir):
     hosts_file = tmpdir.mkdir("etc").join("hosts")
     hosts_file.write("6.6.6.6\texample.com\n")
     hosts = Hosts(path=hosts_file.strpath)
-    assert(repr(hosts)) == "Hosts(hosts_path='{0}', " \
-                           "entries=[HostsEntry(entry_type='ipv4', " \
-                           "address='6.6.6.6', " \
-                           "comment=None, " \
-                           "names=['example.com'])])".format(hosts_file.strpath)
+    assert (repr(hosts)) == "Hosts(hosts_path='{0}', " \
+                            "entries=[HostsEntry(entry_type='ipv4', " \
+                            "address='6.6.6.6', " \
+                            "comment=None, " \
+                            "names=['example.com'])])".format(hosts_file.strpath)
 
 
 def test_import_from_url_counters_for_part_success(tmpdir):
@@ -53,14 +73,15 @@ def test_exception_raised_when_unable_to_write_hosts(tmpdir):
     """ Test that the correct exception is raised when a hosts file
     is not writeable.
     """
-    hosts_file = tmpdir.mkdir("etc").join("hosts")
-    hosts_file.write("127.0.0.1\tlocalhost\n")
-    hosts = Hosts(path=hosts_file.strpath)
-    os.chmod(hosts_file.strpath, 0o444)
-    new_entry = HostsEntry(entry_type='ipv4', address='123.123.123.123', names=['test.example.com'])
-    hosts.add(entries=[new_entry])
-    with pytest.raises(exception.UnableToWriteHosts):
-        hosts.write()
+    if get_username() != 'root':
+        hosts_file = tmpdir.mkdir("etc").join("hosts")
+        hosts_file.write("127.0.0.1\tlocalhost\n")
+        hosts = Hosts(path=hosts_file.strpath)
+        os.chmod(hosts_file.strpath, 0o444)
+        new_entry = HostsEntry(entry_type='ipv4', address='123.123.123.123', names=['test.example.com'])
+        hosts.add(entries=[new_entry])
+        with pytest.raises(exception.UnableToWriteHosts):
+            hosts.write()
 
 
 def test_write_will_create_path_if_missing():
@@ -402,7 +423,7 @@ def test_no_entries_if_hosts_path_does_not_exist():
     Test that no entries are returned if the hosts path is invalid
     """
     hosts = Hosts(path="invalid")
-    assert len(hosts.entries) == 0
+    assert hosts.count() == 0
 
 
 def test_line_break_identified_as_blank(tmpdir):
@@ -471,10 +492,26 @@ def test_file_import_fails_when_not_readable(tmpdir):
 
 
 def test_remove_all_matching_multiple(tmpdir):
+    """ 
+    Test removal of multiple entries with a common alias
+    """
     hosts_file = tmpdir.mkdir("etc").join("hosts")
     hosts_file.write("1.2.3.4\tfoo-1 foo\n"
                      "2.3.4.5\tfoo-2 foo\n")
     hosts = Hosts(path=hosts_file.strpath)
     hosts.remove_all_matching(name="foo")
     hosts.write()
-    assert hosts_file.read() == ""
+    assert not hosts_file.read()
+
+
+def test_remove_all_matching_failure(tmpdir):
+    """
+    Test removal of multiple entries with a common alias
+    """
+    with pytest.raises(ValueError):
+        hosts_file = tmpdir.mkdir("etc").join("hosts")
+        hosts_file.write("1.2.3.4\tfoo-1 foo\n"
+                         "2.3.4.5\tfoo-2 foo\n")
+        hosts = Hosts(path=hosts_file.strpath)
+        hosts.remove_all_matching()
+        hosts.write()
